
/*
 * libgnet - game net library
 * Copyright (C) 1997 Indrek Mandre 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <sys/un.h>
#include <string.h>
#include <errno.h>

#include "gnet.h"
#include "gnet-local.h"

char GN_error[256];
char *GN_unix_socket_name = NULL;

void GN_set_unix_socket_path ( char *path )
{

 if ( GN_unix_socket_name != NULL ) free ( GN_unix_socket_name );

 if ( path == NULL ) return;

 GN_unix_socket_name = malloc ( strlen ( path ) );

 strcpy ( GN_unix_socket_name, path );

}

GN_server * GN_init_server ( int sock_type, int port, int sock_arg )
{
 GN_server *serv;

   switch ( sock_type ) {

   	case NET_UNIX:
   	{
   	    int s;
   	    struct sockaddr_un sunix;
   	    int len = sizeof ( struct sockaddr_un );

   		if ( sock_arg & CON_SEQPACKET ) {
   			sprintf ( GN_error, "Unix socket doesn't need CON_SEQPACKET\n");
   			return NULL;
		}

		if ( sock_arg & CON_UDP ) {
			sprintf ( GN_error, "Unix socket doesn't support UDP protocol\n");
			return NULL;
		}

		if ( GN_unix_socket_name == NULL ) {
			sprintf ( sunix.sun_path, "/tmp/GN%d", port );
		} else {
			strncpy ( sunix.sun_path, GN_unix_socket_name, UNIX_PATH_MAX );
		}

		sunix.sun_family = AF_UNIX;

		unlink ( sunix.sun_path );	/* UNLINK it, security??? */

		s = socket ( AF_UNIX, SOCK_STREAM, 0 );

		if ( s < 0 ) {
			sprintf ( GN_error, "Error creating AF_UNIX socket, errno=%s\n", strerror ( errno ) );
			return NULL;
		}

		if ( bind ( s, (struct  sockaddr  *)&sunix, len ) == -1 ) {
			sprintf ( GN_error, "Error binding AF_UNIX socket to file %s, errno=%s\n", sunix.sun_path, strerror ( errno ) );
			return NULL;
		}

		if ( listen ( s, 5 ) == -1 ) {
			sprintf ( GN_error, "Error listening AF_UNIX socket, errno=%s\n", strerror ( errno ) );
			return NULL;
		}

		serv = malloc ( sizeof ( GN_server ) );

		serv->tcpsocket = s;
		serv->tcpsocktype = sock_type;
		serv->tcpsockarg = sock_arg;
		serv->isudp = 0;

		GN_init_clients ( serv );

		chmod ( sunix.sun_path, 0777 );	/* Make socket world write/readable */

		return serv;

   		break;
	}

	case NET_INTERNET:
	{
   	    int s;
   	    struct sockaddr_in sinet;
   	    int len = sizeof ( struct sockaddr_in );
   	    int x = 1;

   	    	   memset ( &sinet, 0, sizeof ( struct sockaddr_in ) );

   	    	   sinet.sin_family = AF_INET;
   	    	   sinet.sin_addr.s_addr = INADDR_ANY;
   	    	   sinet.sin_port = htons ( port );

   	    	   if ( sock_arg & CON_SEQPACKET )
	   	    	   s = socket ( AF_INET, SOCK_SEQPACKET, 0 );
	   	    else   s = socket ( AF_INET, SOCK_STREAM, 0 );

	   	   setsockopt( s, SOL_SOCKET, SO_REUSEADDR,(char *) &x, sizeof(x) );

   	    	   if ( s < 0 ) {
			sprintf ( GN_error, "Error creating AF_INET socket, %s protocol, errno=%s\n", (sock_arg & CON_SEQPACKET)?"SOCK_SEQPACKET":"SOCK_STREAM", strerror ( errno ) );
			return NULL;
		   }

		   if ( bind ( s, (struct sockaddr *)&sinet, len ) == -1 ) {
			sprintf ( GN_error, "Error binding AF_INET socket to port %d, %s protocol, errno=%s\n", port, (sock_arg & CON_SEQPACKET)?"SOCK_SEQPACKET":"SOCK_STREAM", strerror ( errno ) );
			return NULL;
		   }

		   if ( listen ( s, 5 ) == -1 ) {
			sprintf ( GN_error, "Error listening AF_INET socket, %s protocol, errno=%s\n", (sock_arg & CON_SEQPACKET)?"SOCK_SEQPACKET":"SOCK_STREAM", strerror ( errno ) );
			return NULL;
		   }

   	    	   serv = malloc ( sizeof ( GN_server ) );

		   serv->tcpsocket = s;
		   serv->tcpsocktype = sock_type;
		   serv->tcpsockarg = sock_arg;
   	    	   if ( sock_arg & CON_UDP )
   	    	   	serv->isudp = 1;
   	    	   	  else serv->isudp = 0;

		   GN_init_clients ( serv );

   	    	return serv;
		break;
	}

	default:

		sprintf ( GN_error, "Unknown socket type %d\n", sock_type );
		return NULL;
		break;
   }

}

void GN_close_server ( GN_server *server )
{

 if ( server->tcpsocket != -1 ) close ( server->tcpsocket );

 while ( server->clients->prev != NULL ) {
 	if ( server->clients->udpaddress != NULL ) free ( server->clients->udpaddress );
 	close ( server->clients->tcpsocket );
 	if ( server->clients->udpsocket != -1 ) close ( server->clients->udpsocket );
 	server->clients = server->clients->prev;
 	free ( server->clients->next );
 }
	if ( server->clients->udpaddress != NULL ) free ( server->clients->udpaddress );
	close ( server->clients->tcpsocket );
	if ( server->clients->udpsocket != -1 ) close ( server->clients->udpsocket );
	free ( server->clients );

 close ( server->tcpsocket );

 free ( server );

}

